
import { localize } from '@codingame/monaco-vscode-api/vscode/vs/nls';
import { deepClone } from '@codingame/monaco-vscode-api/vscode/vs/base/common/objects';
import schema$1 from './jsonSchemaCommon.js';
import { ProblemMatcherRegistry } from './problemMatcher.js';
import { TaskDefinitionRegistry } from '@codingame/monaco-vscode-api/vscode/vs/workbench/contrib/tasks/common/taskDefinitionRegistry';
import { applyDeprecatedVariableMessage } from '@codingame/monaco-vscode-2448342b-297f-5823-9f41-f18eef0f6ee8-common/vscode/vs/workbench/services/configurationResolver/common/configurationResolverUtils';
import { inputsSchema } from '@codingame/monaco-vscode-2448342b-297f-5823-9f41-f18eef0f6ee8-common/vscode/vs/workbench/services/configurationResolver/common/configurationResolverSchema';
import { getAllCodicons } from '@codingame/monaco-vscode-api/vscode/vs/base/common/codicons';

function fixReferences(literal) {
    if (Array.isArray(literal)) {
        literal.forEach(fixReferences);
    }
    else if (typeof literal === 'object') {
        if (literal['$ref']) {
            literal['$ref'] = literal['$ref'] + '2';
        }
        Object.getOwnPropertyNames(literal).forEach(property => {
            const value = literal[property];
            if (Array.isArray(value) || typeof value === 'object') {
                fixReferences(value);
            }
        });
    }
}
const shellCommand = {
    anyOf: [
        {
            type: 'boolean',
            default: true,
            description: ( localize(
                9197,
                'Specifies whether the command is a shell command or an external program. Defaults to false if omitted.'
            ))
        },
        {
            $ref: '#/definitions/shellConfiguration'
        }
    ],
    deprecationMessage: ( localize(
        9198,
        'The property isShellCommand is deprecated. Use the type property of the task and the shell property in the options instead. See also the 1.14 release notes.'
    ))
};
const hide = {
    type: 'boolean',
    description: ( localize(9199, 'Hide this task from the run task quick pick')),
    default: true
};
const taskIdentifier = {
    type: 'object',
    additionalProperties: true,
    properties: {
        type: {
            type: 'string',
            description: ( localize(9200, 'The task identifier.'))
        }
    }
};
const dependsOn = {
    anyOf: [
        {
            type: 'string',
            description: ( localize(9201, 'Another task this task depends on.'))
        },
        taskIdentifier,
        {
            type: 'array',
            description: ( localize(9202, 'The other tasks this task depends on.')),
            items: {
                anyOf: [
                    {
                        type: 'string',
                    },
                    taskIdentifier
                ]
            }
        }
    ],
    description: ( localize(
        9203,
        'Either a string representing another task or an array of other tasks that this task depends on.'
    ))
};
const dependsOrder = {
    type: 'string',
    enum: ['parallel', 'sequence'],
    enumDescriptions: [
        ( localize(9204, 'Run all dependsOn tasks in parallel.')),
        ( localize(9205, 'Run all dependsOn tasks in sequence.')),
    ],
    default: 'parallel',
    description: ( localize(
        9206,
        'Determines the order of the dependsOn tasks for this task. Note that this property is not recursive.'
    ))
};
const detail = {
    type: 'string',
    description: ( localize(
        9207,
        'An optional description of a task that shows in the Run Task quick pick as a detail.'
    ))
};
const icon = {
    type: 'object',
    description: ( localize(9208, 'An optional icon for the task')),
    properties: {
        id: {
            description: ( localize(9209, 'An optional codicon ID to use')),
            type: ['string', 'null'],
            enum: Array.from(getAllCodicons(), icon => icon.id),
            markdownEnumDescriptions: Array.from(getAllCodicons(), icon => `$(${icon.id})`),
        },
        color: {
            description: ( localize(9210, 'An optional color of the icon')),
            type: ['string', 'null'],
            enum: [
                'terminal.ansiBlack',
                'terminal.ansiRed',
                'terminal.ansiGreen',
                'terminal.ansiYellow',
                'terminal.ansiBlue',
                'terminal.ansiMagenta',
                'terminal.ansiCyan',
                'terminal.ansiWhite'
            ],
        },
    }
};
const presentation = {
    type: 'object',
    default: {
        echo: true,
        reveal: 'always',
        focus: false,
        panel: 'shared',
        showReuseMessage: true,
        clear: false,
    },
    description: ( localize(
        9211,
        'Configures the panel that is used to present the task\'s output and reads its input.'
    )),
    additionalProperties: false,
    properties: {
        echo: {
            type: 'boolean',
            default: true,
            description: ( localize(
                9212,
                'Controls whether the executed command is echoed to the panel. Default is true.'
            ))
        },
        focus: {
            type: 'boolean',
            default: false,
            description: ( localize(
                9213,
                'Controls whether the panel takes focus. Default is false. If set to true the panel is revealed as well.'
            ))
        },
        revealProblems: {
            type: 'string',
            enum: ['always', 'onProblem', 'never'],
            enumDescriptions: [
                ( localize(9214, 'Always reveals the problems panel when this task is executed.')),
                ( localize(9215, 'Only reveals the problems panel if a problem is found.')),
                ( localize(9216, 'Never reveals the problems panel when this task is executed.')),
            ],
            default: 'never',
            description: ( localize(
                9217,
                'Controls whether the problems panel is revealed when running this task or not. Takes precedence over option \"reveal\". Default is \"never\".'
            ))
        },
        reveal: {
            type: 'string',
            enum: ['always', 'silent', 'never'],
            enumDescriptions: [
                ( localize(9218, 'Always reveals the terminal when this task is executed.')),
                ( localize(
                9219,
                'Only reveals the terminal if the task exits with an error or the problem matcher finds an error.'
            )),
                ( localize(9220, 'Never reveals the terminal when this task is executed.')),
            ],
            default: 'always',
            description: ( localize(
                9221,
                'Controls whether the terminal running the task is revealed or not. May be overridden by option \"revealProblems\". Default is \"always\".'
            ))
        },
        panel: {
            type: 'string',
            enum: ['shared', 'dedicated', 'new'],
            default: 'shared',
            description: ( localize(
                9222,
                'Controls if the panel is shared between tasks, dedicated to this task or a new one is created on every run.'
            ))
        },
        showReuseMessage: {
            type: 'boolean',
            default: true,
            description: ( localize(
                9223,
                'Controls whether to show the `Terminal will be reused by tasks, press any key to close it` message.'
            ))
        },
        clear: {
            type: 'boolean',
            default: false,
            description: ( localize(
                9224,
                'Controls whether the terminal is cleared before executing the task.'
            ))
        },
        group: {
            type: 'string',
            description: ( localize(
                9225,
                'Controls whether the task is executed in a specific terminal group using split panes.'
            ))
        },
        close: {
            type: 'boolean',
            description: ( localize(
                9226,
                'Controls whether the terminal the task runs in is closed when the task exits.'
            ))
        }
    }
};
const terminal = deepClone(presentation);
terminal.deprecationMessage = ( localize(9227, 'The terminal property is deprecated. Use presentation instead'));
const groupStrings = {
    type: 'string',
    enum: [
        'build',
        'test',
        'none'
    ],
    enumDescriptions: [
        ( localize(
        9228,
        'Marks the task as a build task accessible through the \'Run Build Task\' command.'
    )),
        ( localize(
        9229,
        'Marks the task as a test task accessible through the \'Run Test Task\' command.'
    )),
        ( localize(9230, 'Assigns the task to no group'))
    ],
    description: ( localize(9231, 'The task\'s execution group.'))
};
const group = {
    oneOf: [
        groupStrings,
        {
            type: 'object',
            properties: {
                kind: groupStrings,
                isDefault: {
                    type: ['boolean', 'string'],
                    default: false,
                    description: ( localize(
                        9232,
                        'Defines if this task is the default task in the group, or a glob to match the file which should trigger this task.'
                    ))
                }
            }
        },
    ],
    defaultSnippets: [
        {
            body: { kind: 'build', isDefault: true },
            description: ( localize(9233, 'Marks the task as the default build task.'))
        },
        {
            body: { kind: 'test', isDefault: true },
            description: ( localize(9234, 'Marks the task as the default test task.'))
        }
    ],
    description: ( localize(
        9235,
        'Defines to which execution group this task belongs to. It supports "build" to add it to the build group and "test" to add it to the test group.'
    ))
};
const taskType = {
    type: 'string',
    enum: ['shell'],
    default: 'process',
    description: ( localize(
        9236,
        'Defines whether the task is run as a process or as a command inside a shell.'
    ))
};
const command = {
    oneOf: [
        {
            oneOf: [
                {
                    type: 'string'
                },
                {
                    type: 'array',
                    items: {
                        type: 'string'
                    },
                    description: ( localize(
                        9237,
                        'The shell command to be executed. Array items will be joined using a space character'
                    ))
                }
            ]
        },
        {
            type: 'object',
            required: ['value', 'quoting'],
            properties: {
                value: {
                    oneOf: [
                        {
                            type: 'string'
                        },
                        {
                            type: 'array',
                            items: {
                                type: 'string'
                            },
                            description: ( localize(
                                9237,
                                'The shell command to be executed. Array items will be joined using a space character'
                            ))
                        }
                    ],
                    description: ( localize(9238, 'The actual command value'))
                },
                quoting: {
                    type: 'string',
                    enum: ['escape', 'strong', 'weak'],
                    enumDescriptions: [
                        ( localize(
                        9239,
                        'Escapes characters using the shell\'s escape character (e.g. ` under PowerShell and \\ under bash).'
                    )),
                        ( localize(
                        9240,
                        'Quotes the argument using the shell\'s strong quote character (e.g. \' under PowerShell and bash).'
                    )),
                        ( localize(
                        9241,
                        'Quotes the argument using the shell\'s weak quote character (e.g. " under PowerShell and bash).'
                    )),
                    ],
                    default: 'strong',
                    description: ( localize(9242, 'How the command value should be quoted.'))
                }
            }
        }
    ],
    description: ( localize(
        9243,
        'The command to be executed. Can be an external program or a shell command.'
    ))
};
const args = {
    type: 'array',
    items: {
        oneOf: [
            {
                type: 'string',
            },
            {
                type: 'object',
                required: ['value', 'quoting'],
                properties: {
                    value: {
                        type: 'string',
                        description: ( localize(9244, 'The actual argument value'))
                    },
                    quoting: {
                        type: 'string',
                        enum: ['escape', 'strong', 'weak'],
                        enumDescriptions: [
                            ( localize(
                            9239,
                            'Escapes characters using the shell\'s escape character (e.g. ` under PowerShell and \\ under bash).'
                        )),
                            ( localize(
                            9240,
                            'Quotes the argument using the shell\'s strong quote character (e.g. \' under PowerShell and bash).'
                        )),
                            ( localize(
                            9241,
                            'Quotes the argument using the shell\'s weak quote character (e.g. " under PowerShell and bash).'
                        )),
                        ],
                        default: 'strong',
                        description: ( localize(9245, 'How the argument value should be quoted.'))
                    }
                }
            }
        ]
    },
    description: ( localize(9246, 'Arguments passed to the command when this task is invoked.'))
};
const label = {
    type: 'string',
    description: ( localize(9247, "The task's user interface label"))
};
const version = {
    type: 'string',
    enum: ['2.0.0'],
    description: ( localize(9248, 'The config\'s version number.'))
};
const identifier = {
    type: 'string',
    description: ( localize(
        9249,
        'A user defined identifier to reference the task in launch.json or a dependsOn clause.'
    )),
    deprecationMessage: ( localize(
        9250,
        'User defined identifiers are deprecated. For custom task use the name as a reference and for tasks provided by extensions use their defined task identifier.'
    ))
};
const runOptions = {
    type: 'object',
    additionalProperties: false,
    properties: {
        reevaluateOnRerun: {
            type: 'boolean',
            description: ( localize(9251, 'Whether to reevaluate task variables on rerun.')),
            default: true
        },
        runOn: {
            type: 'string',
            enum: ['default', 'folderOpen'],
            description: ( localize(
                9252,
                'Configures when the task should be run. If set to folderOpen, then the task will be run automatically when the folder is opened.'
            )),
            default: 'default'
        },
        instanceLimit: {
            type: 'number',
            description: ( localize(
                9253,
                'The number of instances of the task that are allowed to run simultaneously.'
            )),
            default: 1
        },
    },
    description: ( localize(9254, 'The task\'s run related options'))
};
const commonSchemaDefinitions = schema$1.definitions;
const options = deepClone(commonSchemaDefinitions.options);
const optionsProperties = options.properties;
optionsProperties.shell = deepClone(commonSchemaDefinitions.shellConfiguration);
const taskConfiguration = {
    type: 'object',
    additionalProperties: false,
    properties: {
        label: {
            type: 'string',
            description: ( localize(9255, "The task's label"))
        },
        taskName: {
            type: 'string',
            description: ( localize(9256, 'The task\'s name')),
            deprecationMessage: ( localize(
                9257,
                'The task\'s name property is deprecated. Use the label property instead.'
            ))
        },
        identifier: deepClone(identifier),
        group: deepClone(group),
        isBackground: {
            type: 'boolean',
            description: ( localize(
                9258,
                'Whether the executed task is kept alive and is running in the background.'
            )),
            default: true
        },
        promptOnClose: {
            type: 'boolean',
            description: ( localize(
                9259,
                'Whether the user is prompted when VS Code closes with a running task.'
            )),
            default: false
        },
        presentation: deepClone(presentation),
        icon: deepClone(icon),
        hide: deepClone(hide),
        options: options,
        problemMatcher: {
            $ref: '#/definitions/problemMatcherType',
            description: ( localize(
                9260,
                'The problem matcher(s) to use. Can either be a string or a problem matcher definition or an array of strings and problem matchers.'
            ))
        },
        runOptions: deepClone(runOptions),
        dependsOn: deepClone(dependsOn),
        dependsOrder: deepClone(dependsOrder),
        detail: deepClone(detail),
    }
};
const taskDefinitions = [];
TaskDefinitionRegistry.onReady().then(() => {
    updateTaskDefinitions();
});
function updateTaskDefinitions() {
    for (const taskType of TaskDefinitionRegistry.all()) {
        if (taskDefinitions.find(schema => {
            return schema.properties?.type?.enum?.find ? schema.properties?.type.enum.find(element => element === taskType.taskType) : undefined;
        })) {
            continue;
        }
        const schema = deepClone(taskConfiguration);
        const schemaProperties = schema.properties;
        schemaProperties.type = {
            type: 'string',
            description: ( localize(9261, 'The task type to customize')),
            enum: [taskType.taskType]
        };
        if (taskType.required) {
            schema.required = taskType.required.slice();
        }
        else {
            schema.required = [];
        }
        schema.required.push('type');
        if (taskType.properties) {
            for (const key of ( Object.keys(taskType.properties))) {
                const property = taskType.properties[key];
                schemaProperties[key] = deepClone(property);
            }
        }
        fixReferences(schema);
        taskDefinitions.push(schema);
    }
}
const customize = deepClone(taskConfiguration);
customize.properties.customize = {
    type: 'string',
    deprecationMessage: ( localize(
        9262,
        'The customize property is deprecated. See the 1.14 release notes on how to migrate to the new task customization approach'
    ))
};
if (!customize.required) {
    customize.required = [];
}
customize.required.push('customize');
taskDefinitions.push(customize);
const definitions = deepClone(commonSchemaDefinitions);
const taskDescription = definitions.taskDescription;
taskDescription.required = ['label'];
const taskDescriptionProperties = taskDescription.properties;
taskDescriptionProperties.label = deepClone(label);
taskDescriptionProperties.command = deepClone(command);
taskDescriptionProperties.args = deepClone(args);
taskDescriptionProperties.isShellCommand = deepClone(shellCommand);
taskDescriptionProperties.dependsOn = dependsOn;
taskDescriptionProperties.hide = deepClone(hide);
taskDescriptionProperties.dependsOrder = dependsOrder;
taskDescriptionProperties.identifier = deepClone(identifier);
taskDescriptionProperties.type = deepClone(taskType);
taskDescriptionProperties.presentation = deepClone(presentation);
taskDescriptionProperties.terminal = terminal;
taskDescriptionProperties.icon = deepClone(icon);
taskDescriptionProperties.group = deepClone(group);
taskDescriptionProperties.runOptions = deepClone(runOptions);
taskDescriptionProperties.detail = detail;
taskDescriptionProperties.taskName.deprecationMessage = ( localize(
    9257,
    'The task\'s name property is deprecated. Use the label property instead.'
));
const processTask = deepClone(taskDescription);
taskDescription.default = {
    label: 'My Task',
    type: 'shell',
    command: 'echo Hello',
    problemMatcher: []
};
definitions.showOutputType.deprecationMessage = ( localize(
    9263,
    'The property showOutput is deprecated. Use the reveal property inside the presentation property instead. See also the 1.14 release notes.'
));
taskDescriptionProperties.echoCommand.deprecationMessage = ( localize(
    9264,
    'The property echoCommand is deprecated. Use the echo property inside the presentation property instead. See also the 1.14 release notes.'
));
taskDescriptionProperties.suppressTaskName.deprecationMessage = ( localize(
    9265,
    'The property suppressTaskName is deprecated. Inline the command with its arguments into the task instead. See also the 1.14 release notes.'
));
taskDescriptionProperties.isBuildCommand.deprecationMessage = ( localize(
    9266,
    'The property isBuildCommand is deprecated. Use the group property instead. See also the 1.14 release notes.'
));
taskDescriptionProperties.isTestCommand.deprecationMessage = ( localize(
    9267,
    'The property isTestCommand is deprecated. Use the group property instead. See also the 1.14 release notes.'
));
processTask.properties.type = {
    type: 'string',
    enum: ['process'],
    default: 'process',
    description: ( localize(
        9236,
        'Defines whether the task is run as a process or as a command inside a shell.'
    ))
};
processTask.required.push('command');
processTask.required.push('type');
taskDefinitions.push(processTask);
taskDefinitions.push({
    $ref: '#/definitions/taskDescription'
});
const definitionsTaskRunnerConfigurationProperties = definitions.taskRunnerConfiguration.properties;
const tasks = definitionsTaskRunnerConfigurationProperties.tasks;
tasks.items = {
    oneOf: taskDefinitions
};
definitionsTaskRunnerConfigurationProperties.inputs = inputsSchema.definitions.inputs;
definitions.commandConfiguration.properties.isShellCommand = deepClone(shellCommand);
definitions.commandConfiguration.properties.args = deepClone(args);
definitions.options.properties.shell = {
    $ref: '#/definitions/shellConfiguration'
};
definitionsTaskRunnerConfigurationProperties.isShellCommand = deepClone(shellCommand);
definitionsTaskRunnerConfigurationProperties.type = deepClone(taskType);
definitionsTaskRunnerConfigurationProperties.group = deepClone(group);
definitionsTaskRunnerConfigurationProperties.presentation = deepClone(presentation);
definitionsTaskRunnerConfigurationProperties.suppressTaskName.deprecationMessage = ( localize(
    9265,
    'The property suppressTaskName is deprecated. Inline the command with its arguments into the task instead. See also the 1.14 release notes.'
));
definitionsTaskRunnerConfigurationProperties.taskSelector.deprecationMessage = ( localize(
    9268,
    'The property taskSelector is deprecated. Inline the command with its arguments into the task instead. See also the 1.14 release notes.'
));
const osSpecificTaskRunnerConfiguration = deepClone(definitions.taskRunnerConfiguration);
delete osSpecificTaskRunnerConfiguration.properties.tasks;
osSpecificTaskRunnerConfiguration.additionalProperties = false;
definitions.osSpecificTaskRunnerConfiguration = osSpecificTaskRunnerConfiguration;
definitionsTaskRunnerConfigurationProperties.version = deepClone(version);
const schema = {
    oneOf: [
        {
            'allOf': [
                {
                    type: 'object',
                    required: ['version'],
                    properties: {
                        version: deepClone(version),
                        windows: {
                            '$ref': '#/definitions/osSpecificTaskRunnerConfiguration',
                            'description': ( localize(9269, 'Windows specific command configuration'))
                        },
                        osx: {
                            '$ref': '#/definitions/osSpecificTaskRunnerConfiguration',
                            'description': ( localize(9270, 'Mac specific command configuration'))
                        },
                        linux: {
                            '$ref': '#/definitions/osSpecificTaskRunnerConfiguration',
                            'description': ( localize(9271, 'Linux specific command configuration'))
                        }
                    }
                },
                {
                    $ref: '#/definitions/taskRunnerConfiguration'
                }
            ]
        }
    ]
};
schema.definitions = definitions;
function deprecatedVariableMessage(schemaMap, property) {
    const mapAtProperty = schemaMap[property].properties;
    if (mapAtProperty) {
        ( Object.keys(mapAtProperty)).forEach(name => {
            deprecatedVariableMessage(mapAtProperty, name);
        });
    }
    else {
        applyDeprecatedVariableMessage(schemaMap[property]);
    }
}
Object.getOwnPropertyNames(definitions).forEach(key => {
    const newKey = key + '2';
    definitions[newKey] = definitions[key];
    delete definitions[key];
    deprecatedVariableMessage(definitions, newKey);
});
fixReferences(schema);
function updateProblemMatchers() {
    try {
        const matcherIds = ( ( ProblemMatcherRegistry.keys()).map(key => '$' + key));
        definitions.problemMatcherType2.oneOf[0].enum = matcherIds;
        definitions.problemMatcherType2.oneOf[2].items.anyOf[0].enum = matcherIds;
    }
    catch (err) {
        console.log('Installing problem matcher ids failed');
    }
}
ProblemMatcherRegistry.onReady().then(() => {
    updateProblemMatchers();
});

export { schema as default, updateProblemMatchers, updateTaskDefinitions };
